# Generated by Django 3.2.12 on 2022-03-08 01:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cjtmc_base', '0003_env_siteid'),
    ]

    operations = [
        migrations.AddField(
            model_name='idc',
            name='Remark',
            field=models.CharField(blank=True, max_length=100, null=True, verbose_name='备注信息'),
        ),
    ]



# !/usr/bin/env python
# coding: utf-8
# @Time    : 2020/10/19 08:59
# @Author  : derek
# @File    : regions.py
# @Version : 1.0
# 说明:
import json

from aliyunsdkcore.client import AcsClient
from aliyunsdkecs.request.v20140526.DescribeRegionsRequest import DescribeRegionsRequest

from cjtmc_base.models import Region
from cjtmc_base.sdk.ali.base import AliBase


class Regions(AliBase):

    def get_region(self):
        """
        获取region
        :return:
        """
        client = AcsClient(self.AccessKeyId, self.AccessSecret)
        request = DescribeRegionsRequest()
        request.set_accept_format('json')
        try:
            response = json.loads(client.do_action_with_exception(request).decode('utf-8'))
            for region in response.get('Regions').get('Region'):
                if region:
                    region['OriginId'] = self.Origin
                    Region.objects.get_or_create(**region)
            return True
        except Exception as e:
            return False



import json
import logging
import math
import random
import time

from aliyunsdkcore.client import AcsClient

from cjtmc_base.utils.rsa import decrypt

logger = logging.getLogger('cjtmc_base.utils.log.tasks')


class AliBase():

    def __init__(self, Origin=None, RegionId='cn-hangzhou'):
        self.Origin = Origin
        self.AccessKeyId = Origin.AccessKeyId
        self.AccessSecret = decrypt(Origin.AccessSecret)
        self.RegionId = RegionId
        self.client = AcsClient(self.AccessKeyId, self.AccessSecret, self.RegionId, timeout=100)

    def get_filed_name(self, model):
        """
        获取model的字段名称用于校验
        :param model:
        :return:
        """
        return [field.name for field in model._meta.get_fields() if field.name != 'id']

    def edit_base(self, modelname=None, param={}):
        request = modelname()
        request.set_accept_format('json')
        for k, v in param.items():
            request.add_query_param(k, str(v).strip())
        logger.info("模块为{},参数为{}".format(modelname, request.get_query_params()))
        response = self.get_response(request)
        return response

    def get_base(self, modelname=None, param=None):
        """
        通过分页的方式获取分页信息
        """
        if param is None: param = {}
        request = modelname()
        request.set_accept_format('json')
        request.add_query_param('PageSize', 50)
        request.add_query_param('CurrentPage', 1)
        for p, v in param.items():
            request.add_query_param(p, str(v).strip())
        response = self.get_response(request).get('message')
        pagenum = "PageNumber"
        if 'TotalCount' in response:
            count = response['TotalCount']
        # 不要删,采集Arms需要定制的页参数
        elif 'PageBean' in response and modelname.__name__ in ("SearchAlertRulesRequest"):
            count = response.get('PageBean').get('TotalCount')
            pagenum = "CurrentPage"
        elif 'TotalRecordCount' in response:
            count = response.get('TotalRecordCount')
        elif 'TotalItems' in response:
            count = response.get('TotalItems')
        elif 'PageBean' in response:
            count = response.get('PageBean').get('TotalCount')
        elif 'Data' in response and modelname.__name__ in (
                "QueryInstanceBillRequest", "DescribeSplitItemBillRequest", "QueryCostUnitRequest",
                "QueryCostUnitResourceRequest"):  # 获取账单数据的多嵌套了一层，所以需要get 2次
            count = response.get("Data").get("TotalCount")
            pagenum = "PageNum"
        elif modelname.__name__ in ('DescribeAlarmEventListRequest'):
            pagenum = "CurrentPage"
            count = response.get('PageInfo').get('TotalCount')
        else:
            count = 100
        for page in range(1, math.ceil(count / 50) + 1):
            request.add_query_param(pagenum, page)
            response = self.get_response(request)
            yield response.get('message', {})

    def get_base_by_token(self, modelname=None, param=None) -> dict:
        """
        通过token方式分页获取信息
        """
        if not param: param = {}
        request = modelname()
        request.set_accept_format('json')
        request.add_query_param('MaxResults', 50)  # 每页条数
        for p, v in param.items():
            # 这里p是执行方法，查看阿里云sdk源码发现，"set_方法名"到了这里就得变成 "方法名",举个例子，查询账单是request.set_BillingCycle，到了这里就得写成 BillingCycle
            request.add_query_param(p, str(v).strip())
        response = self.get_response(request)
        # response.get('NextToken')是云主机、磁盘信息等获取token的方式
        # response.get("Data", {}).get('NextToken')是分账账单获取token的方式
        # 如果有特殊情况，自行在后面添加token获取方式
        token = response.get('message').get('NextToken') or response.get('message').get("Data", {}).get('NextToken')
        yield response.get('message', {})
        while token:
            request.add_query_param('MaxResults', 50)  # 每页条数
            for p, v in param.items():
                # 这里p是执行方法，查看阿里云sdk源码发现，"set_方法名"到了这里就得变成 "方法名",举个例子，查询账单是request.set_BillingCycle，到了这里就得写成 BillingCycle
                request.add_query_param(p, str(v).strip())
            request.add_query_param("NextToken", token)
            response = self.get_response(request)
            token = response.get('message').get('NextToken') or response.get('message').get("Data", {}).get('NextToken')
            yield response.get('message', {})

    def get_response(self, request):
        """根据请求返回处理后的响应信息"""
        try:
            response = {'code': 200,
                        'message': json.loads(self.client.do_action_with_exception(request).decode('utf-8'))}
        except Exception as e:
            response = {'code': 500, 'message': str(e)}
            logger.error('连接阿里云失败，错误代码为{}'.format(e))
            RETRY_MSGS = ['Throttling', 'LastTokenProcessing', 'SYSTEM.CONCURRENT_OPERATE']
            # 只要任意一个可以触发重试的信息在str(e)存在，则重试
            if any([retry_msg in str(e) for retry_msg in RETRY_MSGS]):
                time.sleep(random.randint(60, 60 * 5))
                logger.info('限流后重试....')
                response = {'code': 200,
                            'message': json.loads(self.client.do_action_with_exception(request).decode('utf-8'))
                            }

        return response



#!/usr/bin/env python
"""
Written by nickcooper-zhangtonghao
Github: https://github.com/nickcooper-zhangtonghao
Email: nickcooper-zhangtonghao@opencloud.tech

Note: Example code For testing purposes only

This code has been released under the terms of the Apache-2.0 license
http://opensource.org/licenses/Apache-2.0
"""
from pyVmomi import vim
from . import cli, pchelper, service_instance


def add_nic(si, vm, network_name):
    """
    :param si: Service Instance
    :param vm: Virtual Machine Object
    :param network_name: Name of the Virtual Network
    """
    spec = vim.vm.ConfigSpec()
    nic_changes = []

    nic_spec = vim.vm.device.VirtualDeviceSpec()
    nic_spec.operation = vim.vm.device.VirtualDeviceSpec.Operation.add

    nic_spec.device = vim.vm.device.VirtualE1000()

    nic_spec.device.deviceInfo = vim.Description()
    nic_spec.device.deviceInfo.summary = 'vCenter API test'

    content = si.RetrieveContent()
    network = pchelper.get_obj(content, [vim.Network], network_name)
    if isinstance(network, vim.OpaqueNetwork):
        nic_spec.device.backing = \
            vim.vm.device.VirtualEthernetCard.OpaqueNetworkBackingInfo()
        nic_spec.device.backing.opaqueNetworkType = \
            network.summary.opaqueNetworkType
        nic_spec.device.backing.opaqueNetworkId = \
            network.summary.opaqueNetworkId
    else:
        nic_spec.device.backing = \
            vim.vm.device.VirtualEthernetCard.NetworkBackingInfo()
        nic_spec.device.backing.useAutoDetect = False
        nic_spec.device.backing.deviceName = network_name

    nic_spec.device.connectable = vim.vm.device.VirtualDevice.ConnectInfo()
    nic_spec.device.connectable.startConnected = True
    nic_spec.device.connectable.allowGuestControl = True
    nic_spec.device.connectable.connected = False
    nic_spec.device.connectable.status = 'untried'
    nic_spec.device.wakeOnLanEnabled = True
    nic_spec.device.addressType = 'assigned'

    nic_changes.append(nic_spec)
    spec.deviceChange = nic_changes
    vm.ReconfigVM_Task(spec=spec)
    print("NIC CARD ADDED")


def main():
    """
    Sample for adding a NIC to vm
    """
    parser = cli.Parser()
    parser.add_required_arguments(cli.Argument.PORT_GROUP)
    parser.add_optional_arguments(cli.Argument.VM_NAME, cli.Argument.UUID)
    args = parser.get_args()
    si = service_instance.connect(args)

    vm = None
    if args.uuid:
        search_index = si.content.searchIndex
        vm = search_index.FindByUuid(None, args.uuid, True)
    elif args.vm_name:
        content = si.RetrieveContent()
        vm = pchelper.get_obj(content, [vim.VirtualMachine], args.vm_name)

    if vm:
        add_nic(si, vm, args.port_group)
    else:
        print("VM not found")


# start this thing
if __name__ == "__main__":
    main()



# Generated by Django 3.2.12 on 2022-03-07 02:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cjtmc_base', '0002_env'),
    ]

    operations = [
        migrations.AddField(
            model_name='env',
            name='SiteId',
            field=models.IntegerField(blank=True, default=1, null=True, verbose_name='租户id'),
        ),
    ]



import logging
from uuid import uuid4

import requests
from django.conf import settings

from cjtmc_base.utils.rsa import decrypt

logger = logging.getLogger("cjtmc_base.utils.log.tasks")


class ZstackBase(object):
    byte2G = lambda self, x: int(x / 1024 / 1024 / 1024)
    uuid = property(fget=lambda x: "".join(str(uuid4()).split("-")))

    def __init__(self, *args, **kwargs):
        self.Origin = kwargs.get("Origin")
        self.name = self.Origin.AccessKeyId
        self.password = decrypt(self.Origin.AccessSecret)
        self.region = kwargs.get("RegionId", "zone-1")
        host, port = self.Origin.Settings.get("Ip"), self.Origin.Settings.get("Port")
        self.base_url = f"http://{host}:{port}"

    def login(self):
        """
        登录:获取一个Session UUID，以供后续API调用使用
        """
        login_data = {"logInByAccount": {"password": self.password, "accountName": self.name}}
        res = requests.post(self.base_url + "/zstack/v1/accounts/login", json=login_data).json()
        return res.get("inventory", {}).get("uuid")

    def logout(self, uuid):
        """
        登出:否则会报错(登录会话数量已经达到最大值)
        """
        return requests.post(self.base_url + f"/zstack/v1/accounts/sessions/{uuid}").json()

    def execute(self, url, data={}, method="get", web_hook=False):
        """
        执行zstack请求的核心代码
        :param url: 网址
        :param data: 数据
        :param method: 执行方法
        :param web_hook: 是否异步回调
        :return: 响应数据，字典
        """
        # logger.info(f"zstack base req=", url, data, method, web_hook)
        # 登录
        uuid = self.login()
        if not getattr(settings, 'ZSTACK_CALLBACK', ''):
            logger.error('请在配置文件中配置ZSTACK_CALLBACK回调网址')
            settings.ZSTACK_CALLBACK = 'http://www.xxx.com/api/callback/'
        # 核心操作
        extend_headers = {"X-Job-UUID": self.uuid, "X-Web-Hook": settings.ZSTACK_CALLBACK}
        headers = {"Authorization": "OAuth {}".format(uuid), "Content-Type": "application/json"}
        if web_hook: headers.update(**extend_headers)
        if method == "get":
            response = requests.get(url=self.base_url + url, headers=headers, params=data).json()
        else:
            response = requests.request(url=self.base_url + url, headers=headers, json=data, method=method).json()
        # 登出
        self.logout(uuid)
        return response

    def edit_base(self, url, data, method="post", web_hook=False):
        """
        执行功能封装
        """
        try:
            res = self.execute(url, data, method, web_hook)
        except Exception as e:
            response = {"code": 500, "msg": str(e), "data": []}
        else:
            response = {"code": 200, "msg": "执行成功", "data": res.get('inventory')}
        return response

    def get_base(self, url, data=None, pageSize=100):
        """
        获取功能封装(分页采集,默认每页100条)
        """
        if not data: data = {"start": 0, "limit": pageSize, "replyWithCount": True}
        # 数据总数，用于确定分页
        total = self.execute(url, data, "get").get("total", 0)

        pages = int(total // pageSize) + 1
        for page in range(pages):
            logger.info(f"page={page},url={url}采集数据")
            data["start"] = page * pageSize
            results = self.execute(url, data, 'get').get("inventories", {})
            yield from results



# !/usr/bin/env python
# coding: utf-8
# @Time    : 2020/10/29 10:30
# @Author  : derek
# @File    : regions.py
# @Version : 1.0
# 说明:
import logging

from huaweicloudsdkcore.auth.credentials import GlobalCredentials
from huaweicloudsdkcore.http.http_config import HttpConfig
from huaweicloudsdkiam.v3 import IamClient, KeystoneListProjectsRequest

from cjtmc_base.models import Region
from cjtmc_base.sdk.huawei.base import HuaweiBase

logger = logging.getLogger("cjtmc_base.utils.log.tasks")


class Regions(HuaweiBase):

    def get_region(self, *args, **kwargs):
        endpoint = "https://iam.myhuaweicloud.com"
        config = HttpConfig.get_default_config()
        config.timeout = 3
        config.ignore_ssl_verification = True
        credentials = GlobalCredentials(self.AccessKeyId, self.AccessSecret)

        client = IamClient.new_builder(IamClient).with_http_config(config). \
            with_credentials(credentials). \
            with_endpoint(endpoint).build()

        request = KeystoneListProjectsRequest()
        try:
            projects = client.keystone_list_projects(request)
        except Exception as e:
            return False
        project_dict = {
            'cn-northeast-1': "东北-大连",
            'cn-north-4': '华北-北京四',
            "cn-north-1": "华北-北京一",
            'cn-east-2': '华东-上海二',
            'cn-east-3': '华东-上海一',
            'cn-south-1': '华南-广州',
            'af-south-1': '南非-约翰内斯堡',
            'eu-west-0': '欧洲-巴黎',
            'ap-southeast-1': '亚太-香港',
            'ap-southeast-3': '亚太-新加坡',
            'cn-southwest-2': '西南-贵阳一',
            'ap-southeast-2': '亚太-曼谷'
        }

        logger.info("-----数据为{}----".format(projects))
        for project in projects.projects:
            project = project.to_dict()
            if project.get("name") != 'MOS':
                Region.objects.get_or_create(RegionId=project.get("name"), RegionEndpoint=project.get("id"),
                                             OriginId=self.Origin,
                                             LocalName=project_dict.get(project.get("name"), project.get("name")))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/7/7 19:19
# @Author  : Derek
# @Version : V0.1
# @function:

class FileClassNotFoundError(Exception):
    def __init__(self):
        pass

    def __str__(self):
        return ('当前云平台没有此文件，不能进行操作')


class InstanceNotFoundError(Exception):
    def __init__(self):
        pass

    def __str__(self):
        return ('当前机器未找到，请重新查询')



"""chanjetmsp URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/3.0/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from rest_framework.routers import DefaultRouter

from cjtmc_base.base.views import OrginAPIView, IdcViewSet, EnvAPIView

urlpatterns = [
]

route = DefaultRouter()
route.register('origin', OrginAPIView, basename="origin")
route.register('idc', IdcViewSet, basename="idc")
route.register('env', EnvAPIView, basename="env")
urlpatterns += route.urls



# !/usr/bin/env python
# coding: utf-8
# @Time    : 2020/9/8 17:23
# @Author  : derek
# @File    : log_handler.py
# @Version : 1.0
# 说明:
import fcntl
import os
import re
import time
import datetime
import logging
from logging.handlers import TimedRotatingFileHandler

try:
    import codecs
except ImportError:
    codecs = None


class MultiprocessHandler(logging.FileHandler):
    """支持多进程的TimedRotatingFileHandler"""

    def __init__(self, filename, when='D', backupCount=0, encoding=None, delay=False):
        """filename 日志文件名,when 时间间隔的单位,backupCount 保留文件个数
        delay 是否开启 OutSteam缓存
            True 表示开启缓存，OutStream输出到缓存，待缓存区满后，刷新缓存区，并输出缓存数据到文件。
            False表示不缓存，OutStrea直接输出到文件"""
        self.prefix = filename
        self.backupCount = backupCount
        self.when = when.upper()
        # 正则匹配 年-月-日
        self.extMath = r"^\d{4}-\d{2}-\d{2}"

        # S 每秒建立一个新文件
        # M 每分钟建立一个新文件
        # H 每小时建立一个新文件
        # D 每天建立一个新文件
        # MIDNIGHT 每天建立一个新文件
        self.when_dict = {
            'S': "%Y-%m-%d-%H-%M-%S",
            'M': "%Y-%m-%d-%H-%M",
            'H': "%Y-%m-%d-%H",
            'D': "%Y-%m-%d",
            'MIDNIGHT': "%Y-%m-%d"
        }
        # 日志文件日期后缀
        self.suffix = self.when_dict.get(when)
        if not self.suffix:
            raise ValueError("指定的日期间隔单位无效: %s" % self.when)
        # 拼接文件路径 格式化字符串
        self.filefmt = os.path.join("logs", "%s.%s" % (self.prefix, self.suffix))
        # 使用当前时间，格式化文件格式化字符串
        self.filePath = datetime.datetime.now().strftime(self.filefmt)
        # 获得文件夹路径
        _dir = os.path.dirname(self.filefmt)
        try:
            # 如果日志文件夹不存在，则创建文件夹
            if not os.path.exists(_dir):
                os.makedirs(_dir)
        except Exception:
            print("创建文件夹失败")
            print(("文件夹路径：" + self.filePath))
            pass

        if codecs is None:
            encoding = None

        logging.FileHandler.__init__(self, self.filePath, 'a+', encoding, delay)

    def shouldChangeFileToWrite(self):
        """更改日志写入目的写入文件
        :return True 表示已更改，False 表示未更改"""
        # 以当前时间获得新日志文件路径
        _filePath = datetime.datetime.now().strftime(self.filefmt)
        # 新日志文件日期 不等于 旧日志文件日期，则表示 已经到了日志切分的时候
        #   更换日志写入目的为新日志文件。
        # 例如 按 天 （D）来切分日志
        #   当前新日志日期等于旧日志日期，则表示在同一天内，还不到日志切分的时候
        #   当前新日志日期不等于旧日志日期，则表示不在
        # 同一天内，进行日志切分，将日志内容写入新日志内。
        if _filePath != self.filePath:
            self.filePath = _filePath
            return True
        return False

    def doChangeFile(self):
        """输出信息到日志文件，并删除多于保留个数的所有日志文件"""
        # 日志文件的绝对路径
        self.baseFilename = os.path.abspath(self.filePath)
        # stream == OutStream
        # stream is not None 表示 OutStream中还有未输出完的缓存数据
        if self.stream:
            # flush close 都会刷新缓冲区，flush不会关闭stream，close则关闭stream
            # self.stream.flush()
            self.stream.close()
            # 关闭stream后必须重新设置stream为None，否则会造成对已关闭文件进行IO操作。
            self.stream = None
        # delay 为False 表示 不OutStream不缓存数据 直接输出
        #   所有，只需要关闭OutStream即可
        if not self.delay:
            # 这个地方如果关闭colse那么就会造成进程往已关闭的文件中写数据，从而造成IO错误
            # delay == False 表示的就是 不缓存直接写入磁盘
            # 我们需要重新在打开一次stream
            # self.stream.close()
            self.stream = self._open()
        # 删除多于保留个数的所有日志文件
        if self.backupCount > 0:
            print('删除日志')
            for s in self.getFilesToDelete():
                print(s)
                os.remove(s)

    def getFilesToDelete(self):
        """获得过期需要删除的日志文件"""
        # 分离出日志文件夹绝对路径
        # split返回一个元组（absFilePath,fileName)
        # 例如：split('I:\ScripPython\char4\mybook\util\logs\mylog.2017-03-19）
        # 返回（I:\ScripPython\char4\mybook\util\logs， mylog.2017-03-19）
        # _ 表示占位符，没什么实际意义，
        dirName, _ = os.path.split(self.baseFilename)
        fileNames = os.listdir(dirName)
        result = []
        # self.prefix 为日志文件名 列如：mylog.2017-03-19 中的 mylog
        # 加上 点号 . 方便获取点号后面的日期
        prefix = self.prefix + '.'
        plen = len(prefix)
        for fileName in fileNames:
            if fileName[:plen] == prefix:
                # 日期后缀 mylog.2017-03-19 中的 2017-03-19
                suffix = fileName[plen:]
                # 匹配符合规则的日志文件，添加到result列表中
                if re.compile(self.extMath).match(suffix):
                    result.append(os.path.join(dirName, fileName))
        result.sort()

        # 返回  待删除的日志文件
        #   多于 保留文件个数 backupCount的所有前面的日志文件。
        if len(result) < self.backupCount:
            result = []
        else:
            result = result[:len(result) - self.backupCount]
        return result

    def emit(self, record):
        """发送一个日志记录
        覆盖FileHandler中的emit方法，logging会自动调用此方法"""
        try:
            if self.shouldChangeFileToWrite():
                self.doChangeFile()
            logging.FileHandler.emit(self, record)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)


class SafeFileHandler(logging.FileHandler):
    def __init__(self, filename, mode, encoding=None, delay=0):
        """
        Use the specified filename for streamed logging
        """
        if codecs is None:
            encoding = None
        logging.FileHandler.__init__(self, filename, mode, encoding, delay)
        self.mode = mode
        self.encoding = encoding
        self.suffix = "%Y-%m-%d"
        self.suffix_time = ""

    def emit(self, record):
        """
        Emit a record.

        Always check time
        """
        try:
            if self.check_baseFilename(record):
                self.build_baseFilename()
            logging.FileHandler.emit(self, record)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

    def check_baseFilename(self, record):
        """
        Determine if builder should occur.

        record is not used, as we are just comparing times,
        but it is needed so the method signatures are the same
        """
        timeTuple = time.localtime()

        if self.suffix_time != time.strftime(self.suffix, timeTuple) or not os.path.exists(
                self.baseFilename + '.' + self.suffix_time):
            return 1
        else:
            return 0

    def build_baseFilename(self):
        """
        do builder; in this case,
        old time stamp is removed from filename and
        a new time stamp is append to the filename
        """
        if self.stream:
            self.stream.close()
            self.stream = None

        # remove old suffix
        if self.suffix_time != "":
            index = self.baseFilename.find("." + self.suffix_time)
            if index == -1:
                index = self.baseFilename.rfind(".")
            self.baseFilename = self.baseFilename[:index]

        # add new suffix
        currentTimeTuple = time.localtime()
        self.suffix_time = time.strftime(self.suffix, currentTimeTuple)
        self.baseFilename = self.baseFilename + "." + self.suffix_time

        self.mode = 'a'
        if not self.delay:
            self.stream = self._open()


class MultiCompatibleTimedRotatingFileHandler(TimedRotatingFileHandler):

    def doRollover(self):
        if self.stream:
            self.stream.close()
            self.stream = None
        # get the time that this sequence started at and make it a TimeTuple
        currentTime = int(time.time())
        dstNow = time.localtime(currentTime)[-1]
        t = self.rolloverAt - self.interval
        if self.utc:
            timeTuple = time.gmtime(t)
        else:
            timeTuple = time.localtime(t)
            dstThen = timeTuple[-1]
            if dstNow != dstThen:
                if dstNow:
                    addend = 3600
                else:
                    addend = -3600
                timeTuple = time.localtime(t + addend)
        dfn = self.baseFilename + "." + time.strftime(self.suffix, timeTuple)
        # 兼容多进程并发 LOG_ROTATE
        if not os.path.exists(dfn):
            f = open(self.baseFilename, 'a')
            fcntl.lockf(f.fileno(), fcntl.LOCK_EX)
            if os.path.exists(self.baseFilename):
                os.rename(self.baseFilename, dfn)
        if self.backupCount > 0:
            for s in self.getFilesToDelete():
                os.remove(s)
        if not self.delay:
            self.stream = self._open()
        newRolloverAt = self.computeRollover(currentTime)
        while newRolloverAt <= currentTime:
            newRolloverAt = newRolloverAt + self.interval
        # If DST changes and midnight or weekly rollover, adjust for this.
        if (self.when == 'MIDNIGHT' or self.when.startswith('W')) and not self.utc:
            dstAtRollover = time.localtime(newRolloverAt)[-1]
            if dstNow != dstAtRollover:
                if not dstNow:  # DST kicks in before next rollover, so we need to deduct an hour
                    addend = -3600
                else:  # DST bows out before next rollover, so we need to add an hour
                    addend = 3600
                newRolloverAt += addend
        self.rolloverAt = newRolloverAt



# Generated by Django 3.2.12 on 2022-04-15 01:43

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cjtmc_base', '0005_auto_20220414_0853'),
    ]

    operations = [
        migrations.AddField(
            model_name='origin',
            name='Default',
            field=models.BooleanField(default=False, verbose_name='是否为默认账号'),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/6/9 20:28
# @Author  : Derek
# @Version : V0.1
# @function:
from collections import OrderedDict

from django.http import Http404
from rest_framework import exceptions
from rest_framework.exceptions import PermissionDenied
from rest_framework.pagination import PageNumberPagination
from rest_framework.response import Response
from rest_framework.views import set_rollback
from rest_framework.viewsets import ModelViewSet


class PageNumber(PageNumberPagination):
    page_size = 10
    page_size_query_param = 'pageSize'  # URL中每页显示条数的参数
    page_query_param = 'pageNumber'  # URL中页码的参数
    max_page_size = None  # 最大页码数限制

    def get_paginated_response(self, data):
        return Response(OrderedDict([
            ('total', self.page.paginator.count),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('list', data),
        ]))


class CustomModelViewSet(ModelViewSet):
    """重写ModelViewSet确保返回信息为{'code': xxx, 'msg': '成功', 'errors': {}, 'data': []} """
    pagination_class = PageNumber

    def create(self, request, *args, **kwargs):
        print(self.request.query_params)
        response = super(CustomModelViewSet, self).create(request, *args, **kwargs)
        return Response({'code': 200, "message": "创建成功", "data": response.data})

    def retrieve(self, request, *args, **kwargs):
        response = super(CustomModelViewSet, self).retrieve(request, *args, **kwargs)
        # response.data, headers=response.status_code
        return Response({"message": "查看详情成功", "code": 200, "data": response.data})

    def update(self, request, *args, **kwargs):
        response = super(CustomModelViewSet, self).update(request, *args, **kwargs)
        return Response({'code': 200, "message": "更新成功", "data": response.data})

    def destroy(self, request, *args, **kwargs):
        response = super(CustomModelViewSet, self).destroy(request, *args, **kwargs)
        return Response({'code': 200, "message": "删除成功", "data": response.data})

    def list(self, request, *args, **kwargs):
        response = super(CustomModelViewSet, self).list(request, *args, **kwargs)
        return Response({'code': 200, "message": "查看成功", "data": response.data})


# 异常处理机制
def exception_handler(exc, context):
    """
    Returns the response that should be used for any given exception.

    By default we handle the REST framework `APIException`, and also
    Django's built-in `Http404` and `PermissionDenied` exceptions.

    Any unhandled exceptions may return `None`, which will cause a 500 error
    to be raised.
    """
    if isinstance(exc, Http404):
        exc = exceptions.NotFound()
    elif isinstance(exc, PermissionDenied):
        exc = exceptions.PermissionDenied()

    if isinstance(exc, exceptions.APIException):
        headers = {}
        if getattr(exc, 'auth_header', None):
            headers['WWW-Authenticate'] = exc.auth_header
        if getattr(exc, 'wait', None):
            headers['Retry-After'] = '%d' % exc.wait

        if isinstance(exc.detail, (list, dict)):
            data = exc.detail
        else:
            data = exc.detail
        set_rollback()
        return Response({'code': 500, "msg": '失败原因:' + str(data), "errors": data, 'data': []},
                        status=exc.status_code,
                        headers=headers)
    return None



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/12/15 15:01
# @Author  : Derek
# @Version : V0.1
# @function:
LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'standard': {
            'format': '%(asctime)s [%(threadName)s:%(thread)d] [%(name)s:%(lineno)d] [%(module)s:%(funcName)s] [%(levelname)s]- %(message)s'}
    },
    'filters': {
        'require_debug_true': {  # 只有在setting中的 DEBUG = True 的时候才会生效
            '()': 'django.utils.log.RequireDebugTrue',
        }, },
    'handlers': {
        'error': {
            'level': 'ERROR',
            'class': 'utils.log_handler.MultiCompatibleTimedRotatingFileHandler',
            'filename': '/var/log/multicloud/error.log',
            'when': 'MIDNIGHT',
            'encoding': 'utf-8',
            'formatter': 'standard',
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'standard',
            'filters': ['require_debug_true']
        },
        'tasks_handler': {
            'level': 'DEBUG',
            'class': 'utils.log_handler.MultiCompatibleTimedRotatingFileHandler',
            'filename': '/var/log/multicloud/tasks.log',
            'when': 'MIDNIGHT',
            'backupCount': 5,
            'formatter': 'standard',
            'encoding': 'utf-8',
        },
    },
    'loggers': {
        'tasks': {
            'handlers': ['tasks_handler', 'error'],
            'level': 'DEBUG',
            'propagate': True,
        },
    }
}



# !/usr/bin/env python
# coding: utf-8
# @Time    : 2020/10/29 08:55
# @Author  : derek
# @File    : base.py
# @Version : 1.0
# 说明:

from huaweicloudsdkcore.auth.credentials import BasicCredentials
from huaweicloudsdkcore.auth.credentials import GlobalCredentials
from huaweicloudsdkcore.http.http_config import HttpConfig

from cjtmc_base.utils.rsa import decrypt


class HuaweiBase():

    def __init__(self, Origin=None, RegionId='cn-north-4'):
        self.Origin = Origin
        self.AccessKeyId = Origin.AccessKeyId
        self.AccessSecret = decrypt(Origin.AccessSecret)
        self.region = RegionId  # cn-north-4
        self.RegionId = str("_".join(RegionId.split("-"))).upper() if RegionId else ""  # CN_NORTH_4
        print(self.Origin, self.AccessKeyId)

    def get_filed_name(self, model):
        """
        获取model的字段名称用于校验
        :param model:
        :return:
        """
        return [field.name for field in model._meta.get_fields() if field.name != 'id']

    @property
    def config(self):
        config = HttpConfig.get_default_config()
        config.timeout = 100
        config.ignore_ssl_verification = True
        return config

    @property
    def basecredentials(self):
        return BasicCredentials(self.AccessKeyId, self.AccessSecret)

    @property
    def globalcredentials(self):
        return GlobalCredentials(self.AccessKeyId, self.AccessSecret)


from functools import wraps
import logging

logger = logging.getLogger("tasks")


def edit_base(fun):
    """用于统一返回值，如果失败返回500状态码"""

    @wraps(fun)
    def wrapper(*args, **kwargs):
        try:
            msg = fun(*args, **kwargs)
            response = {'code': 200, 'message': '执行成功', 'data': str(msg)}
        except Exception as e:
            logger.error('连接云sdk失败，错误代码为{}'.format(e))
            response = {'code': 500, 'message': str(e)}  # e转成字符串，便于返回报错信息
        return response

    return wrapper



-----BEGIN PUBLIC KEY-----
MIGfMA0GCSqGSIb3DQEBAQUAA4GNADCBiQKBgQDIFK4ncX8wQfJO4hgxNpv1E/BX
OT4D8mlylTRn69Z/cjenDFIQhSuvDff+1c6qPRG5UH29iIafXYGYFanTJUBULrxJ
M7BgXPpZRLPI+ESy+v5kgIwE42XEbMstJzhDiu+cpj/2QT84fEy62ZnC4hCphgF+
DlnZu7pA2ykZ/Pr6swIDAQAB
-----END PUBLIC KEY-----



# !/usr/bin/env python
# coding: utf-8
# @Time    : 2020/11/30 10:14
# @Author  : derek
# @File    : region.py
# @Version : 1.0
# 说明:
import logging

from cjtmc_base.models import Region
from cjtmc_base.sdk.vmware.base import VmwareBase

logger = logging.getLogger("cjtmc_base.utils.log.tasks")


class Regions(VmwareBase):
    def get_region(self):
        logger.info("----同步vmware可用区域开始----")
        Region.objects.update_or_create(defaults={'RegionId': "vmware", 'LocalName': "vmware", 'OriginId': self.Origin},
                                        RegionId="vmware"
                                        )
        logger.info("----同步vmware可用区域结束----")



# !/usr/bin/env python
# coding: utf-8
# @Time    : 2020/10/29 08:55
# @Author  : derek
# @File    : base.py
# @Version : 1.0
# 说明:

import logging
import math

from tencentcloud.billing.v20180709.billing_client import BillingClient
from tencentcloud.cbs.v20170312.cbs_client import CbsClient
from tencentcloud.common import credential
from tencentcloud.common.profile.client_profile import ClientProfile
from tencentcloud.common.profile.http_profile import HttpProfile
from tencentcloud.cvm.v20170312.cvm_client import CvmClient
from tencentcloud.vpc.v20170312.vpc_client import *

from cjtmc_base.utils.rsa import decrypt

logger = logging.getLogger("cjtmc_base.utils.log.tasks")


class TencentBase(object):
    def __init__(self, *args, **kwargs) -> object:
        """
        出现异常：[TencentCloudSDKException] code:ClientNetworkError message:[SSL: CERTIFICATE_VERIFicate verify failed:
                unable to get local issuer certificate (_ssl.c:1124) requestId:None
        如何解决：/Applications/Python*/Install Certificates.command 或者 pip install --upgrade certifi
        """
        self.Origin = kwargs.get('Origin')
        self.AccessKeyId = kwargs.get('Origin').AccessKeyId
        self.AccessSecret = decrypt(kwargs.get('Origin').AccessSecret)
        self.RegionId = kwargs.get('RegionId')
        self.cred = credential.Credential(self.AccessKeyId, self.AccessSecret)

    def get_base(self, modelreq, params=None, endpoint='ecs', clinetreq=None):
        """
        通过分页的方式获取分页信息(默认每页100条数据),
        """
        if params is None: params = {}
        self.config(endpoint)
        # logger.info(f'分页获取tencent数据, params={params}')
        req = modelreq()
        if endpoint == 'billing':
            params.update({"Limit": 100, 'Offset': 0})
        else:
            params.update({"limit": 100, 'offset': 0})
        req.from_json_string(json.dumps(params))
        resp = getattr(self.client, clinetreq)(req)
        res = json.loads(resp.to_json_string())
        if 'TotalCount' in res:
            count = res['TotalCount']
        else:
            count = 100
        for page in range(0, math.ceil(count / 100) + 1):
            if endpoint == 'billing':
                params.update({"Limit": 100, 'Offset': page * 100})
            else:
                params.update({"limit": 100, 'offset': page * 100})
            req.from_json_string(json.dumps(params))
            resp = getattr(self.client, clinetreq)(req)
            response = json.loads(resp.to_json_string())
            logger.info("----数据返回，返回结果为{},账号为{}----".format(response, self.AccessKeyId))
            yield response

    def edit_base(self, endpoint='ecs', modelname=None, fname=None, params=None):
        """tencent执行函数的封装,eg:start,stop.reboot..."""
        if not params: params = {}
        self.config(endpoint=endpoint)
        request = modelname()
        request.from_json_string(json.dumps(params))
        fun = getattr(self.client, fname)
        try:
            response = self.new_response(fun(request))
        except Exception as e:
            logger.error(f'腾讯云{self.AccessKeyId}执行{fname}失败,原因:{str(e)}')
            return {'code': 500, 'message': f'执行失败,原因:{str(e)}', 'data': []}
        else:
            return {'code': 200, 'message': response, 'data': response}

    def get_field_name(self, model):
        """
        获取model的字段名称用于校验
        :param model:
        :return:
        """
        return [field.name for field in model._meta.get_fields() if field.name != 'id']

    def clientvcs(self, endpoint=None):
        return getattr(self, f'{endpoint}client')

    def config(self, endpoint=None):
        """添加配置, eg:endpoint=cvm"""
        endpoint2 = endpoint if endpoint != 'region' else 'cvm'
        e = "{}.tencentcloudapi.com".format(endpoint2)  # cvm.tencentcloudapi.com
        self.httpProfile = HttpProfile(endpoint=e, reqTimeout=30)  # 实例化一个 http 选项，可选的，没有特殊需求可以跳过。
        self.clientProfile = ClientProfile(httpProfile=self.httpProfile)  # 实例化一个 client 选项，可选的，没有特殊需求可以跳过。
        self.client = self.clientvcs(endpoint)

    @staticmethod
    def new_response(response):
        """默认返回响应信息为对象,进一步处理返回字典类型"""
        return json.loads(response.to_json_string())

    @property
    def vpcclient(self):
        client = VpcClient(self.cred, self.RegionId, self.clientProfile)
        return client

    @property
    def cbsclient(self):
        client = CbsClient(self.cred, self.RegionId, self.clientProfile)
        return client

    @property
    def cvmclient(self):
        client = CvmClient(self.cred, self.RegionId, self.clientProfile)
        return client

    @property
    def costclient(self):
        client = BillingClient(self.cred, self.RegionId, self.clientProfile)
        return client

    @property
    def regionclient(self):
        client = CvmClient(self.cred, "", self.clientProfile)
        return client

    @property
    def billingclient(self):
        client = BillingClient(self.cred, "", self.clientProfile)
        return client



# !/usr/bin/env python
# coding: utf-8
# @Time    : 2020/10/29 10:35
# @Author  : derek
# @File    : all.py
# @Version : 1.0
# 说明:
import importlib
import logging

from cjtmc_base.utils.error import FileClassNotFoundError

logger = logging.getLogger("cjtmc_base.utils.log.tasks")


class AllBase():
    """
    使用继承来实现，通过重构父进程的方法来实现传参
    """

    def __init__(self, origin, region='cn-beijing', idc='ali'):
        self.idc = idc
        self.origin = origin
        self.region = region

    def client(self, c, name, idc):
        try:
            module = importlib.import_module(f'cjtmc_{name}.sdk.{idc}.{c}')
        except ModuleNotFoundError as e:
            raise FileClassNotFoundError()
        cls_str = str(c).capitalize()
        return getattr(module, cls_str)(**{'Origin': self.origin, 'RegionId': self.region})

    def converse_idc(self, idcname):
        return idcname if idcname not in ['dingding', 'jst', 'jushita'] else 'ali'

    def exec_func(self, clss, func, **kwargs):
        logger.info("执行{}文件的{}函数，具体参数为{}".format(clss, func, kwargs))
        name = 'base' if clss == 'regions' else clss
        cls = self.client(clss, name, self.converse_idc(self.idc))
        if hasattr(cls, func):
            return getattr(cls, func)(**kwargs)
        else:
            raise ModuleNotFoundError()
    # f = AllBase('dsada', 'cn-beijing', 'ali').exec_func('rds', 'start_ecs', **{})
    # print(f)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/7/20 17:25
# @Author  : Derek
# @Version : V0.1
# @function:
from django.db.models.signals import post_migrate
from django_celery_beat.models import CrontabSchedule, PeriodicTask

# 定义receiver函数
from cjtmc_base.models import IDC


def init_idc(sender, **kwargs):
    if sender.name == 'cjtmc_base':
        datas = [{'Name': 'ali', "Idc": '阿里云', "Default": "cn-beijing"},
                 {'Name': 'huawei', "Idc": '华为云', "Default": "cn-north-4"},
                 {'Name': 'tencent', "Idc": '腾讯云', "Default": "cn-beijing"},
                 {'Name': 'openstack', "Idc": 'Openstack', "Default": "cn-beijing"},
                 {"Name": 'zstack', "Idc": 'Zstack', "Default": "cn-beijing"},
                 {"Name": 'ctyun', "Idc": '天翼云', "Default": "cn-beijing"},
                 {"Name": 'dingding', "Idc": '钉钉云', "Default": "cn-beijing"},
                 {"Name": 'jushita', "Idc": '聚石塔', "Default": "cn-beijing"},
                 {"Name": 'kunpeng', "Idc": '鲲鹏', "Default": "cn-beijing"},
                 {"Name": 'jd', "Idc": "京东云", "Default": "cn-beijing"},
                 {"Name": "baidu", "Idc": "百度云", "Default": "cn-beijing"},
                 {"Name": "ucloud", "Idc": "Ucloud", "Default": "cn-beijing"},
                 {"Name": "aws", "Idc": "AWS", "Default": "cn-beijing"},
                 {"Name": "azure", "Idc": "Azure", "Default": "cn-beijing"},
                 {"Name": "qingcloud", "Idc": "青云", "Default": "cn-beijing"},
                 {"Name": "ksyun", "Idc": "金山云", "Default": "cn-beijing"},
                 {"Name": "qiniu", "Idc": '七牛云', "Default": "cn-beijing"},
                 {"Name": "inspur", "Idc": "浪潮云", "Default": "cn-beijing"},
                 {"Name": "yisu", "Idc": "亿速云", "Default": "cn-beijing"},
                 {"Name": "google", "Idc": '谷歌云', "Default": "cn-beijing"},
                 {"Name": "ibm", "Idc": "IBM云", "Default": "cn-beijing"}
                 ]
        for item in datas:
            IDC.objects.update_or_create(defaults=item, Name=item.get("Name"), Idc=item.get("Idc"))
        print('初始化idc数据成功')


def init_task(sender, **kwargs):
    if sender.name == 'cjtmc_base':
        mid, status = CrontabSchedule.objects.get_or_create(minute="0", hour="1", day_of_week='*', day_of_month='1',
                                                            month_of_year='*', timezone='Asia/Shanghai')

        PeriodicTask.objects.get_or_create(name='get_all_regions',
                                           task='cjtmc_base.tasks.resources.get_all_regions',
                                           crontab=mid)
        print('初始化定时任务成功')


post_migrate.connect(init_idc)
post_migrate.connect(init_task)



from functools import wraps
import logging

logger = logging.getLogger("tasks")


def edit_base(fun):
    """用于统一返回值，如果失败返回500状态码"""

    @wraps(fun)
    def wrapper(*args, **kwargs):
        try:
            msg = fun(*args, **kwargs)
            response = {'code': 200, 'message': msg, 'data': str(msg)}
        except Exception as e:
            logger.error('连接云sdk失败，错误代码为{}'.format(e))
            response = {'code': 500, 'message': str(e)}  # e转成字符串，便于返回报错信息
        print(response)
        return response

    return wrapper



import atexit
import logging

import requests
import urllib3
from com.vmware.vcenter_client import VM, Datacenter, ResourcePool, Folder, Datastore, Network
from pyVim.connect import Disconnect
from pyVim.connect import SmartConnect
from vmware.vapi.vsphere.client import create_vsphere_client

from cjtmc_base.models import Origin
from cjtmc_base.utils.rsa import decrypt

logger = logging.getLogger("cjtmc_base.utils.log.tasks")


class VmwareBase():
    def __init__(self, *args, **kwargs):
        default_origin = Origin.objects.filter(IDC__Idc='vmware').first()
        self.Origin = kwargs.get('Origin', default_origin)
        session = requests.session()
        session.verify = False
        urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)
        host = self.Origin.Settings.get('server')
        username = self.Origin.AccessKeyId
        password = decrypt(self.Origin.AccessSecret)
        self.client = create_vsphere_client(server=host, username=username, password=password, session=session)
        self.si = self.connect(host=host, user=username, password=password)
        self.content = self.si.RetrieveContent()

    def connect(self, host, user, password, disable_ssl_verification=True):
        """
        Determine the most preferred API version supported by the specified server,
        then connect to the specified server using that API version, login and return
        the service instance object.
        """
        service_instance = None
        conn_params = dict(host=host, user=user, pwd=password)
        # form a connection...
        try:
            if disable_ssl_verification:
                conn_params['disableSslCertValidation'] = True
                service_instance = SmartConnect(**conn_params)
            else:
                service_instance = SmartConnect(**conn_params)
            # doing this means you don't need to remember to disconnect your script/objects
            atexit.register(Disconnect, service_instance)
        except IOError as io_error:
            logger.error(io_error)
        if not service_instance:
            raise SystemExit("Unable to connect to host with supplied credentials.")
        return service_instance

    @property
    def vmclient(self):
        return self.client.vcenter.VM

    @property
    def powerclient(self):
        return self.client.vcenter.vm.Power

    @property
    def vpclient(self):
        return self.client.vcenter.Network

    @property
    def resourcepool_client(self):
        return self.client.vcenter.ResourcePool

    @property
    def folder_client(self):
        return self.client.vcenter.Folder

    @property
    def niclient(self):
        return self.client.vcenter.vm.hardware.Ethernet

    def get_datacenter(self, datacenter_name):
        filter_spec = Datacenter.FilterSpec(names={datacenter_name})
        datacenter_summaries = self.client.vcenter.Datacenter.list(filter_spec)
        if len(datacenter_summaries) > 0:
            datacenter = datacenter_summaries[0].datacenter
            return datacenter
        else:
            return None

    def get_resource_pool(self, datacenter_name, resource_pool_name=None):
        """
        根据资源池名称返回资源池对象，如果resource_pool_name不存在，则返回该datacenter的第一个资源池对象。
        """
        datacenter = self.get_datacenter(datacenter_name)
        if not datacenter:
            logger.info("Datacenter '{}' not found".format(datacenter_name))
            return None
        names = {resource_pool_name} if resource_pool_name else None
        filter_spec = ResourcePool.FilterSpec(datacenters={datacenter}, names=names)
        resource_pool_summaries = self.resourcepool_client.list(filter_spec)
        if len(resource_pool_summaries) > 0:
            resource_pool = resource_pool_summaries[0].resource_pool
            logger.info("Selecting ResourcePool '{}'".format(resource_pool))
            return resource_pool
        else:
            logger.info("ResourcePool not found in Datacenter '{}'".
                        format(datacenter_name))
            return None

    def get_folder(self, datacenter_name, folder_name):
        datacenter = self.get_datacenter(datacenter_name)
        if not datacenter:
            logger.info("Datacenter '{}' not found".format(datacenter_name))
            return None

        filter_spec = Folder.FilterSpec(type=Folder.Type.VIRTUAL_MACHINE,
                                        names={folder_name},
                                        datacenters={datacenter})

        folder_summaries = self.folder_client.list(filter_spec)
        if len(folder_summaries) > 0:
            folder = folder_summaries[0].folder
            logger.info("Detected folder '{}' as {}".format(folder_name, folder))
            return folder
        else:
            logger.info("Folder '{}' not found".format(folder_name))
            return None

    def get_datastore(self, datacenter_name, datastore_name):
        datacenter = self.get_datacenter(datacenter_name)
        if not datacenter:
            logger.info("Datacenter '{}' not found".format(datacenter_name))
            return None

        filter_spec = Datastore.FilterSpec(names={datastore_name},
                                           datacenters={datacenter})

        datastore_summaries = self.client.vcenter.Datastore.list(filter_spec)
        if len(datastore_summaries) > 0:
            datastore = datastore_summaries[0].datastore
            logger.info("Detected datastore '{}' as {}".format(datacenter_name, datastore))
            return datastore
        else:
            return None

    def get_placement_spec(self, datacenter_name, vm_folder_name, datastore_name):
        """
        Returns a VM placement spec for a resourcepool. Ensures that the
        vm folder and datastore are all in the same datacenter which is specified.
        """
        resource_pool = self.get_resource_pool(datacenter_name)
        folder = self.get_folder(datacenter_name, vm_folder_name)
        datastore = self.get_datastore(datacenter_name, datastore_name)
        placement_spec = VM.PlacementSpec(folder=folder,
                                          resource_pool=resource_pool,
                                          datastore=datastore)
        logger.info("get_placement_spec_for_resource_pool: Result is '{}'".format(placement_spec))
        return placement_spec

    def get_network_backing(self, porggroup_name, datacenter_name, portgroup_type):
        """列出网络"""
        datacenter = self.get_datacenter(datacenter_name)
        if not datacenter:
            logger.info("Datacenter '{}' not found".format(datacenter_name))
            return None

        filter = Network.FilterSpec(datacenters={datacenter},
                                    names={porggroup_name},
                                    types={portgroup_type})
        network_summaries = self.vpclient.list(filter=filter)
        if len(network_summaries) > 0:
            network = network_summaries[0].network
            logger.info("Selecting {} Portgroup Network '{}' ({})".
                        format(portgroup_type, porggroup_name, network))
            return network
        else:
            logger.info("Portgroup Network not found in Datacenter '{}'".
                        format(datacenter_name))
            return None



import logging

from cjtmc_base.models import Region
from cjtmc_base.sdk.openstack.base import OpenstackBase

logger = logging.getLogger("cjtmc_base.utils.log.tasks")


class Regions(OpenstackBase):
    def get_region(self):
        logger.info("----同步OpenStack可用区域开始----")
        Region.objects.update_or_create(
            defaults={'RegionId': "openstack", 'LocalName': "openstack", 'OriginId': self.Origin},
            OriginId=self.Origin,
            RegionId="openstack"
        )
        logger.info("----同步OpenStack可用区域结束----")



import logging

from cjtmc_base.models import Region
from cjtmc_base.sdk.zstack.base import ZstackBase

logger = logging.getLogger("cjtmc_base.utils.log.tasks")


class Regions(ZstackBase):
    def get_region(self):
        try:
            logger.info("----同步Zstack可用区域开始----")
            for zone in self.get_base(url='/zstack/v1/zones'):
                data = {'RegionId': zone.get('name').lower(), 'LocalName': zone.get('name').lower(), 'Sync': True,
                        'OriginId': self.Origin}
                Region.objects.update_or_create(
                    defaults=data, OriginId=self.Origin, RegionId=zone.get('name').lower())
            logger.info("----同步Zstack可用区域结束----")
            return True
        except Exception as e:
            return False

# python try except exception django pandas requests response vmware openstack
# byte isdigit ZeroDivisionError Traceback



from keystoneauth1.identity import v3
from keystoneauth1.session import Session

from cjtmc_base.models import Origin
from cjtmc_base.utils.rsa import decrypt


class OpenstackBase(object):
    def __init__(self, *args, **kwargs):
        default_origin = Origin.objects.filter(IDC__Idc="openstack").first()
        self.Origin = kwargs.get('Origin', default_origin)
        self.username = self.Origin.AccessKeyId
        self.password = decrypt(self.Origin.AccessSecret)
        self.region = kwargs.get('RegionId', 'Default')
        self.auth_url = self.Origin.Settings.get('AuthUrl')
        self.user_domain_name = self.Origin.Settings.get('UserDomainName')
        self.project_domain_name = self.Origin.Settings.get('UserProjectId')

    @property
    def auth(self):
        auth = v3.Password(username=self.username,
                           password=self.password,
                           user_domain_name=self.user_domain_name,
                           project_domain_name=self.project_domain_name,
                           auth_url=self.auth_url)
        return Session(auth=auth)

    def edit_base(self, modelname=None, param=None):
        """
        执行功能封装
        """
        pass

    def get_base(self, modelname=None, param=None):
        """
        获取功能封装
        """
        pass



# !/usr/bin/env python
# coding: utf-8
# @Time    : 2020/7/27 14:48
# @Author  : derek
# @File    : serializer.py
# @Version : 1.0
# 说明:
from rest_framework import serializers

from cjtmc_base.models import Origin, IDC, Env


class OriginSerializer(serializers.ModelSerializer):
    class Meta:
        model = Origin
        exclude = ['AccessSecret']
        depth = 2


class IdcSerializer(serializers.ModelSerializer):
    class Meta:
        model = IDC
        fields = '__all__'


class EnvSerializer(serializers.ModelSerializer):
    class Meta:
        model = Env
        fields = '__all__'



from django.test import TestCase


# Create your tests here.

class IDCTest(TestCase):
    def test_list(self):
        response = self.client.get('/base/idc/')
        self.assertEqual(response.status_code, 200)
        res = response.json()
        print("res", res)
        self.assertEqual(res.get('code'), 200, )

    def test_insert(self):
        data = {"Name": "baidu", "Idc": "百度", "Remark": "测试数据"}
        response = self.client.post('/base/idc/', data=data)
        res = response.json()
        print("in", res)
        self.assertEqual(res.get('code'), 200, )

    def test_update(self):
        data = {"Name": "baidu2"}
        response = self.client.put('/base/idc/1/', data=data, content_type="application/json")
        res = response.json()
        self.assertEqual(res.get('code'), 200, )

    def test_detail(self):
        response = self.client.delete('/base/idc/1/')
        res = response.json()
        self.assertEqual(res.get('code'), 200, )


class ENVTest(TestCase):
    def test_list(self):
        response = self.client.get('/base/env/')
        self.assertEqual(response.status_code, 200)
        res = response.json()
        self.assertEqual(res.get('code'), 200, )

    def test_insert(self):
        data = {"Name": "测试", "Code": "test", "Priority": 10}
        response = self.client.post('/base/env/', data=data)
        res = response.json()
        self.assertEqual(res.get('code'), 200, )

    def test_update(self):
        self.test_insert()
        data = {"Name": "baidu2"}
        response = self.client.put('/base/env/1/', data=data, content_type="application/json")
        res = response.json()
        self.assertEqual(res.get('code'), 200, )

    def test_detail(self):
        self.test_insert()
        response = self.client.delete('/base/env/1/')
        res = response.json()
        self.assertEqual(res.get('code'), 200, )


class RegionTest(TestCase):
    def test_list(self):
        response = self.client.get('/base/env/')
        self.assertEqual(response.status_code, 200)
        res = response.json()
        self.assertEqual(res.get('code'), 200, )

    def test_insert(self):
        data = {"Name": "测试", "Code": "test", "Priority": 10}
        response = self.client.post('/base/env/', data=data)
        res = response.json()
        self.assertEqual(res.get('code'), 200, )

    def test_update(self):
        self.test_insert()
        data = {"Name": "baidu2"}
        response = self.client.put('/base/env/1/', data=data, content_type="application/json")
        res = response.json()
        self.assertEqual(res.get('code'), 200, )

    def test_detail(self):
        self.test_insert()
        response = self.client.delete('/base/env/1/')
        res = response.json()
        self.assertEqual(res.get('code'), 200, )



import base64
import os

from Crypto import Random
from Crypto.Cipher import PKCS1_v1_5 as Cipher_pkcs1_v1_5
from Crypto.PublicKey import RSA

# 伪随机数生成器
random_generator = Random.new().read
# rsa算法生成实例
rsa = RSA.generate(1024, random_generator)
path = os.path.dirname(os.path.abspath(__file__))
print(path)


def encrypt(message, pub_rsa_path='chanjetrskey.py'):
    '''使用公钥加密'''

    def pre_process(msg):
        # base64.b64encode(cipher.encrypt(message[:100].encode()))
        return base64.b64encode(cipher.encrypt(msg.encode()))

    rsa_path = os.path.join(path, pub_rsa_path)
    with open(rsa_path) as f:
        key = f.read()
        rsakey = RSA.importKey(key)
        cipher = Cipher_pkcs1_v1_5.new(rsakey)
        # 长字符串加密报错解决
        if message:
            if len(message) > 100:
                cipher_text = pre_process(message[:100]) + pre_process(message[100:])
            else:
                cipher_text = pre_process(message)
        else:
            cipher_text = None
        return str(cipher_text, encoding='utf-8')


def decrypt(secret_message, rsa_path='chanjetrs.py'):
    '''使用私钥解密'''

    def pre_process(msg):
        # cipher.decrypt(base64.b64decode(secret_message[:174]), random_generator)
        return cipher.decrypt(base64.b64decode(msg), random_generator)

    rsas_path = os.path.join(path, rsa_path)
    print(rsas_path)
    with open(rsas_path) as f:
        key = f.read()
        rsakey = RSA.importKey(key)
        cipher = Cipher_pkcs1_v1_5.new(rsakey)
        if secret_message:
            if len(secret_message) > 172:
                text = pre_process(secret_message[:172]) + pre_process(secret_message[172:])
            else:
                text = pre_process(secret_message)
            return text.decode('utf-8')
        else:
            return None


if __name__ == '__main__':
    passw = "GafVgKkl7Rqkgu0Ra+tee6jDkj+1peE5wEALqQNhY+fAIJjv/4VShH5nwjCSwxrSoWBgPbQQickA/dK49pbuKyQt+TcwJ6BU6CP/ypNrNQRCYbaKhFCm1GM6Aw/ArKZIEdPfRh2dmUN/yO7/AadGrt4Ri63yGfXOmY5LY7aUows="

    ret = decrypt(passw)

    print(ret)



# !/usr/bin/env python
# coding: utf-8
# @Time    : 2020/10/19 08:59
# @Author  : derek
# @File    : regions.py
# @Version : 1.0
# 说明:
import logging

from tencentcloud.common.exception.tencent_cloud_sdk_exception import TencentCloudSDKException
from tencentcloud.cvm.v20170312.models import DescribeRegionsRequest

from cjtmc_base.models import Region
from cjtmc_base.sdk.tencent.base import TencentBase

logger = logging.getLogger("cjtmc_base.utils.log.tasks")


class Regions(TencentBase):

    def get_region(self):
        """
        获取region
        :return:
        """
        logger.info("----同步腾讯云可用区域开始----")
        try:
            for regions in self.get_base(DescribeRegionsRequest, {}, 'region', 'DescribeRegions'):
                for region in regions.get('RegionSet', []):
                    data_region = {'RegionId': region.get('Region'),
                                   'LocalName': region.get('RegionName'),
                                   'OriginId': self.Origin}
                    Region.objects.update_or_create(defaults=data_region, LocalName=region.get('RegionName'))
        except TencentCloudSDKException as err:
            logger.error("----同步腾讯云可用区域失败,失败原因为{}----".format(err))
        logger.info("----同步腾讯云可用区域结束----")



# Generated by Django 3.2.12 on 2022-04-14 08:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('cjtmc_base', '0004_idc_remark'),
    ]

    operations = [
        migrations.AlterField(
            model_name='origin',
            name='Name',
            field=models.CharField(blank=True, db_index=True, max_length=100, null=True, verbose_name='Name'),
        ),
        migrations.AlterField(
            model_name='region',
            name='RegionId',
            field=models.CharField(blank=True, db_index=True, max_length=100, null=True, verbose_name='位置'),
        ),
    ]



# Create your views here.
import logging

from django.db.models import Q
from rest_framework.response import Response

from cjtmc_base.api.all import AllBase
from cjtmc_base.base.serializer import OriginSerializer, IdcSerializer, EnvSerializer
from cjtmc_base.models import Origin, IDC, Env
from cjtmc_base.utils.rsa import encrypt
from cjtmc_base.utils.viewset import CustomModelViewSet

logger = logging.getLogger('cjtmc_base.utils.log.tasks')


class OrginAPIView(CustomModelViewSet):
    """
    list:
    获取列表
    retrieve:
    获取其中一个详情
    update:
    修改
    delete:
    删除
    create:
    创建
    """
    serializer_class = OriginSerializer
    queryset = Origin.objects.all()

    def get_queryset(self):
        keyword = self.request.query_params.get('keyWord', '')
        origin = Origin.objects.all()
        if keyword:
            origin = origin.filter(AccessKeyId__icontains=keyword)
        idc = self.request.query_params.get('idc', '')
        if idc:
            origin = origin.filter(IDC__Name=keyword)
        return origin.all()

    def create(self, request, *args, **kwargs):
        data = request.data
        data["AccessSecret"] = encrypt(data['AccessSecret'])
        if data.get('IDC') == "openstack":
            data['Settings'] = {"AuthUrl": data.get("AuthUrl"), "UserDomainName": data.get("UserDomainName"),
                                "UserProjectId": data.get("UserProjectId")}
        elif data.get('IDC') == "zstack":
            data['Settings'] = {"Ip": data.get("Ip"), "Port": data.get("Port")}
        else:
            data['Settings'] = {}
        data['IDC'] = IDC.objects.filter(Name=data.get('IDC')).first()
        oid = Origin.objects.create(**data)
        try:
            AllBase(origin=oid, idc=data.get('Idc', 'ali')).exec_func('regions', 'get_region')
        except Exception as e:
            logger.exception(e)
            oid.delete()
            return Response({"code": 10001, "message": "添加失败，原因:账号异常，无法采集信息.", })
        return Response({"code": 200, "message": "添加成功"})


class IdcViewSet(CustomModelViewSet):
    serializer_class = IdcSerializer

    def get_queryset(self):
        idc = IDC.objects.all()
        keyword = self.request.query_params.get('keyWord', '')
        if keyword:
            idc = idc.filter(Q(Idc__icontains=keyword) | Q(Name__icontains=keyword))
        return idc.all()


class EnvAPIView(CustomModelViewSet):
    serializer_class = EnvSerializer

    def get_queryset(self):
        keyword = self.request.query_params.get('keyWord', '')
        env = Env.objects.all()
        if keyword:
            env = env.filter(Code__icontains=keyword)
        status = self.request.query_params.get('status', '')
        if status:
            env = env.filter(Status=status)
        return env.all()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2021/7/7 19:22
# @Author  : Derek
# @Version : V0.1
# @function:
from setuptools import setup, find_packages

setup(
    name="cjtmc-base",
    version="0.0.30",
    keywords=["pip", "chanjet_multicloud", "chanjet", "cjtmsp"],
    description="chanjetmulticloud-base sdk",
    long_description="chanjetmulticloud-base sdk for python",
    license="MIT Licence",
    dependency_links=[
        'https://pypi.douban.com/simple',
    ],
    url="https://www.cjtmsp.com",
    author="derek",
    author_email="wangshengh@chanjet.com",

    packages=find_packages(exclude=[
        'ez_setup', 't', 't.*',
    ]),
    include_package_data=True,
    platforms="any",
    zip_safe=False,
    install_requires=[
        "aliyun-python-sdk-core",
        "arrow",
        "celery",
        "huaweicloudsdkcore",
        "huaweicloudsdkiam",
        "mysqlclient",
        "pycryptodome",
        "python-keystoneclient",
        "tencentcloud-sdk-python",
        "djangorestframework",
        "django_celery_beat",
        "Ipy",
    ]
)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/2/24 10:53
# @Author  : Derek
# @Version : V0.1
# @function:
import logging

from celery import shared_task

from cjtmc_base.api.all import AllBase
from cjtmc_base.models import IDC, Origin

logger = logging.getLogger('chanjetmulticloud_base.utils.log.tasks')


def converse_idc(idcname):
    if idcname in ['dingding', 'jst', 'jushita']:
        return 'ali'
    return idcname


@shared_task()
def get_all_regions():
    """采集所有主机的region信息"""
    for idc in IDC.objects.all():
        orgid = Origin.objects.filter(IDC=idc).all()
        for o in orgid:
            logger.info(f'get_all_regions: {idc.Idc}-{o}')
            try:
                idcname = converse_idc(idc.Name)
                AllBase(o, 'cn-beijing', idcname).exec_func('regions', 'get_region')
            except Exception as e:
                logger.error(f'get_all_regions: {idc.Idc}-{o} 采集失败, 原因:{str(e)}')


@shared_task()
def get_all_base(idc, model, cls, region='all', params=None, ak=None):
    """
    封装的函数,支持指定ak和指定region的函数执行
    ak = ['xxx', 'xxxx']
    region = 'cn-beijing'
    """
    if not params: params = {}
    if not ak: ak = []
    orgids = Origin.objects.filter(IDC__Name=idc)
    if ak:
        orgids = orgids.filter(AccessKeyId__in=ak)
    for o in orgids:
        regions = o.region_set.all() if region == 'all' else o.region_set.filter(RegionId=region)
        for r in regions:
            # 不要删, 如果异常，不影响后续的函数执行
            try:
                idc = converse_idc(idc)  # 兼容钉钉云和聚石塔使用ali的sdk接口。
                AllBase(o, r.RegionId, idc).exec_func(cls, model, **params)
            except Exception as e:
                logger.error(f'idc={idc},origin={o} execute {model} failed, reason={str(e)}')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time    : 2022/2/23 11:53
# @Author  : Derek
# @Version : V0.1
# @function:
DBICTSelect = (
    ('Prepaid', '包年包月'),
    ('Postpaid', '按量付费')
)

DBIT = (
    ('Primary', '主实例'),
    ('ReadOnly', '只读实例'),
    ('Guard', '灾备实例'),
    ('Temp', '临时实例'),
)
DBCM = (
    ('Standard', '标准访问模式'),
    ('Performance', '数据库代理模式')
)
DBCate = (
    ('Basic', '基础版'),
    ('HighAvailability', '高可用版'),
    ('Finance', '金融版')
)
ICTSelect = (
    ('PrePaid', '包年包月'),
    ('PostPaid', '按量付费')
)

INCTSelect = (
    ('PayByBandwidth', '按带宽计费'),
    ('PayByTraffic', '按流量计费')
)
CategorySelect = (
    ('cloud', '普通云盘'),
    ('cloud_efficiency', '高效云盘'),
    ('cloud_ssd', 'SSD盘'),
    ('cloud_essd', 'ESSD云盘'),
    ('local_ssd_pro', 'I/O密集型本地盘'),
    ('local_hdd_pro', '吞吐密集型本地盘'),
    ('ephemeral', '本地盘'),
    ('ephemeral_ssd', '本地SSD盘'),
    ('SSD', '超高IO云硬盘'),
    ('GPSSD', '通用型云硬盘'),
    ('SAS', '高IO云硬盘'),
    ('SATA', '普通IO云硬盘'),
    ('CLOUD_BASIC', '普通云盘'),
    ('CLOUD_PREMIUM', '高性能云硬盘'),
    ('CLOUD_SSD', 'SSD云硬盘'),
    ('CLOUD_HSSD', '增强型SSD云硬盘'),
    ('CLOUD_TSSD', '极速型SSD云硬盘'),
)
StoppendMode = (('KeepCharging', "停机后继续收费"), ('StopCharging', "停机后不收费"), ('Not-applicable', '不支持'))

type_choices = (('cve', 'Linux漏洞'), ('sys', 'Windows漏洞'), ('cms', 'WebCMS漏洞'), ('emg', '应急漏洞'), ('app', '应用漏洞'))
status_choices = (
    ('1', "未修复"), ('2', '修复失败'), ('3', '回滚失败'), ('4', '修复中'), ('5', '回滚中'), ('6', '验证中'), ('7', '修复成功'),
    ('8', '修复成功待重启'), ('9', '回滚成功'), ('10', '已忽略'), ('11', '回滚成功待重启'), ('12', '漏洞不存在'), ('20', '已失效'))
necessity_choices = (('asap', '高'), ('later', '中'), ('nntf', '低'))
level_choices = (('serious', '紧急'), ('suspicious', '可疑'), ('remind', '提醒'))



from django.db import models


class BaseModel(models.Model):
    CreateTime = models.DateTimeField('创建时间', auto_now_add=True, )
    ChangeTime = models.DateTimeField('修改时间', auto_now=True)
    Sync = models.BooleanField(default=False)

    class Meta:
        abstract = True
        ordering = ("-CreateTime",)


# Create your models here.
class IDC(models.Model):
    Name = models.CharField('名称', max_length=100, blank=True, null=True)
    Idc = models.CharField('简称', max_length=100, blank=True, null=True)
    Default = models.CharField('默认区域', max_length=100, blank=True, null=True, default='cn-beijing')
    Remark = models.CharField('备注信息', max_length=100, blank=True, null=True)

    def __str__(self):
        return "%s-%s" % (self.Name, self.Idc)


class Origin(BaseModel):
    """
    存放认证信息
    """
    IDC = models.ForeignKey(IDC, on_delete=models.CASCADE, related_name="idc")
    Name = models.CharField('Name', max_length=100, blank=True, null=True, db_index=True)
    AccessKeyId = models.CharField('access Key Id', max_length=100, blank=True, null=True)
    AccessSecret = models.CharField('access Secret', max_length=1000, blank=True, null=True)
    Remark = models.CharField('备注信息', max_length=100, blank=True, null=True)
    Settings = models.JSONField(verbose_name='其他配置信息')
    Default = models.BooleanField('是否为默认账号', default=False)


class Region(BaseModel):
    """
    接入地址
    """
    RegionId = models.CharField('位置', max_length=100, blank=True, null=True, db_index=True)
    RegionEndpoint = models.CharField('地域对应的接入地址', max_length=100, blank=True, null=True)
    LocalName = models.CharField('地域名称', max_length=100, blank=True, null=True)
    OriginId = models.ForeignKey(Origin, on_delete=models.CASCADE, blank=True, null=True, db_index=True)

    def __str__(self):
        return "{}的{}区域".format(self.OriginId.AccessKeyId, self.LocalName)


class Env(BaseModel):
    Name = models.CharField('名字', max_length=100, blank=True, null=True)
    Code = models.CharField("英文标识", max_length=100, blank=True, null=True, unique=True)
    Status = models.BooleanField('环境是否可用', default=1)
    Priority = models.IntegerField('显示优先级', default=0, help_text='优先级递增(0-10)')
    SiteId = models.IntegerField('租户id', blank=True, null=True, default=1)

    class Meta:
        verbose_name = '环境数据表'
        ordering = ('-Priority', 'Name')



#!/usr/bin/env python

# Copyright 2016 Abdul Anshad <abdulanshad33@gmail.com>
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

"""
Written by Abdul Anshad
Github: https://github.com/Abdul-Anshad-A
Email: abdulanshad33@gmail.com

Credits:
Thanks to "reuben.13@gmail.com" for the initial code.

Note: Example code For testing purposes only
vSphere Python SDK program to perform snapshot operations.
"""

import sys
from . import cli, pchelper, service_instance
from pyVmomi import vim
from pyVim.task import WaitForTask


def list_snapshots_recursively(snapshots, count=7):
    snapshot_data = []
    for snapshot in snapshots:
        snapshot_data.append(snapshot)
        snapshot_data = snapshot_data + list_snapshots_recursively(
            snapshot.childSnapshotList)

    if count == 0: return snapshot_data
    return snapshot_data[::-1][:count]


def get_snapshots_by_name_recursively(snapshots, snapname):
    snap_obj = []
    for snapshot in snapshots:
        if snapshot.name == snapname:
            snap_obj.append(snapshot)
        else:
            snap_obj = snap_obj + get_snapshots_by_name_recursively(
                snapshot.childSnapshotList, snapname)
    return snap_obj


def get_current_snap_obj(snapshots, snapob):
    snap_obj = []
    for snapshot in snapshots:
        if snapshot.snapshot == snapob:
            snap_obj.append(snapshot)
        snap_obj = snap_obj + get_current_snap_obj(
            snapshot.childSnapshotList, snapob)
    return snap_obj


def main():
    print("Trying to connect to VCENTER SERVER . . .")
    parser = cli.Parser()
    parser.add_optional_arguments(
        cli.Argument.VM_NAME, cli.Argument.SNAPSHOT_OPERATION, cli.Argument.SNAPSHOT_NAME)
    args = parser.get_args()
    si = service_instance.connect(args)

    print("Connected to VCENTER SERVER !")

    content = si.RetrieveContent()

    vm = pchelper.get_obj(content, [vim.VirtualMachine], args.vm_name)

    if args.snapshot_operation != 'create' and vm.snapshot is None:
        print("Virtual Machine %s doesn't have any snapshots" % vm.name)
        sys.exit()

    if args.snapshot_operation == 'create':
        snapshot_name = args.snapshot_name
        description = "Test snapshot"
        dump_memory = False
        quiesce = False

        print("Creating snapshot %s for virtual machine %s" % (
            snapshot_name, vm.name))
        WaitForTask(vm.CreateSnapshot(
            snapshot_name, description, dump_memory, quiesce))

    elif args.snapshot_operation in ['remove', 'revert']:
        snapshot_name = args.snapshot_name
        snap_obj = get_snapshots_by_name_recursively(
            vm.snapshot.rootSnapshotList, snapshot_name)
        # if len(snap_obj) is 0; then no snapshots with specified name
        if len(snap_obj) == 1:
            snap_obj = snap_obj[0].snapshot
            if args.snapshot_operation == 'remove':
                print("Removing snapshot %s" % snapshot_name)
                WaitForTask(snap_obj.RemoveSnapshot_Task(True))
            else:
                print("Reverting to snapshot %s" % snapshot_name)
                WaitForTask(snap_obj.RevertToSnapshot_Task())
        else:
            print("No snapshots found with name: %s on VM: %s" % (
                snapshot_name, vm.name))

    elif args.snapshot_operation == 'list_all':
        print("Display list of snapshots on virtual machine %s" % vm.name)
        snapshot_paths = list_snapshots_recursively(
            vm.snapshot.rootSnapshotList)
        for snapshot in snapshot_paths:
            print(snapshot)

    elif args.snapshot_operation == 'list_current':
        current_snapref = vm.snapshot.currentSnapshot
        current_snap_obj = get_current_snap_obj(
            vm.snapshot.rootSnapshotList, current_snapref)
        current_snapshot = "Name: %s; Description: %s; " \
                           "CreateTime: %s; State: %s" % (
                               current_snap_obj[0].name,
                               current_snap_obj[0].description,
                               current_snap_obj[0].createTime,
                               current_snap_obj[0].state)
        print("Virtual machine %s current snapshot is:" % vm.name)
        print(current_snapshot)

    elif args.snapshot_operation == 'remove_all':
        print("Removing all snapshots for virtual machine %s" % vm.name)
        WaitForTask(vm.RemoveAllSnapshots())

    else:
        print("Specify operation in "
              "create/remove/revert/list_all/list_current/remove_all")


# Start program
if __name__ == "__main__":
    main()



# Generated by Django 4.0.2 on 2022-02-24 02:51

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):
    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='IDC',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('Name', models.CharField(blank=True, max_length=100, null=True, verbose_name='名称')),
                ('Idc', models.CharField(blank=True, max_length=100, null=True, verbose_name='简称')),
                ('Default',
                 models.CharField(blank=True, default='cn-beijing', max_length=100, null=True, verbose_name='默认区域')),
            ],
        ),
        migrations.CreateModel(
            name='Origin',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('CreateTime', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('ChangeTime', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('Sync', models.BooleanField(default=False)),
                ('Name', models.CharField(blank=True, max_length=100, null=True, verbose_name='Name')),
                ('AccessKeyId', models.CharField(blank=True, max_length=100, null=True, verbose_name='access Key Id')),
                (
                'AccessSecret', models.CharField(blank=True, max_length=1000, null=True, verbose_name='access Secret')),
                ('Remark', models.CharField(blank=True, max_length=100, null=True, verbose_name='备注信息')),
                ('Settings', models.JSONField(verbose_name='其他配置信息')),
                ('IDC', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='idc',
                                          to='cjtmc_base.idc')),
            ],
            options={
                'ordering': ('-CreateTime',),
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Region',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('CreateTime', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('ChangeTime', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('Sync', models.BooleanField(default=False)),
                ('RegionId', models.CharField(blank=True, max_length=100, null=True, verbose_name='位置')),
                ('RegionEndpoint', models.CharField(blank=True, max_length=100, null=True, verbose_name='地域对应的接入地址')),
                ('LocalName', models.CharField(blank=True, max_length=100, null=True, verbose_name='地域名称')),
                ('OriginId', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE,
                                               to='cjtmc_base.origin')),
            ],
            options={
                'ordering': ('-CreateTime',),
                'abstract': False,
            },
        ),
    ]



# Generated by Django 3.2.12 on 2022-03-02 02:04

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('cjtmc_base', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Env',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('CreateTime', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('ChangeTime', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
                ('Sync', models.BooleanField(default=False)),
                ('Name', models.CharField(blank=True, max_length=100, null=True, verbose_name='名字')),
                ('Code', models.CharField(blank=True, max_length=100, null=True, unique=True, verbose_name='英文标识')),
                ('Status', models.BooleanField(default=1, verbose_name='环境是否可用')),
                ('Priority', models.IntegerField(default=0, help_text='优先级递增(0-10)', verbose_name='显示优先级')),
            ],
            options={
                'verbose_name': '环境数据表',
                'ordering': ('-Priority', 'Name'),
            },
        ),
    ]
